@@ -147,6 +147,9 @@ const char *features[] = {
 #if defined(POLARSSL_ENABLE_WEAK_CIPHERSUITES)
     "POLARSSL_ENABLE_WEAK_CIPHERSUITES",
 #endif /* POLARSSL_ENABLE_WEAK_CIPHERSUITES */
+#if defined(POLARSSL_REMOVE_ARC4_CIPHERSUITES)
+    "POLARSSL_REMOVE_ARC4_CIPHERSUITES",
+#endif /* POLARSSL_REMOVE_ARC4_CIPHERSUITES */
 #if defined(POLARSSL_ECP_DP_SECP192R1_ENABLED)
     "POLARSSL_ECP_DP_SECP192R1_ENABLED",
 #endif /* POLARSSL_ECP_DP_SECP192R1_ENABLED */
@@ -336,6 +339,9 @@ const char *features[] = {
 #if defined(POLARSSL_X509_CHECK_EXTENDED_KEY_USAGE)
     "POLARSSL_X509_CHECK_EXTENDED_KEY_USAGE",
 #endif /* POLARSSL_X509_CHECK_EXTENDED_KEY_USAGE */
+#if defined(POLARSSL_X509_RSASSA_PSS_SUPPORT)
+    "POLARSSL_X509_RSASSA_PSS_SUPPORT",
+#endif /* POLARSSL_X509_RSASSA_PSS_SUPPORT */
 #if defined(POLARSSL_ZLIB_SUPPORT)
     "POLARSSL_ZLIB_SUPPORT",
 #endif /* POLARSSL_ZLIB_SUPPORT */
@@ -366,6 +372,9 @@ const char *features[] = {
 #if defined(POLARSSL_CAMELLIA_C)
     "POLARSSL_CAMELLIA_C",
 #endif /* POLARSSL_CAMELLIA_C */
+#if defined(POLARSSL_CCM_C)
+    "POLARSSL_CCM_C",
+#endif /* POLARSSL_CCM_C */
 #if defined(POLARSSL_CERTS_C)
     "POLARSSL_CERTS_C",
 #endif /* POLARSSL_CERTS_C */
