@@ -375,6 +375,7 @@ int main( int argc, char *argv[] )
      */
     server_fd = 0;
     memset( &ssl, 0, sizeof( ssl_context ) );
+    memset( &buf, 0, sizeof( buf ) );
     x509_crt_init( &cacert );
     x509_crt_init( &clicert );
     pk_init( &pkey );
@@ -609,10 +610,18 @@ int main( int argc, char *argv[] )
         ssl_set_ciphersuites( &ssl, opt.force_ciphersuite );
 
     ssl_set_ca_chain( &ssl, &cacert, NULL, opt.server_name );
-    ssl_set_own_cert( &ssl, &clicert, &pkey );
+    if( ( ret = ssl_set_own_cert( &ssl, &clicert, &pkey ) ) != 0 )
+    {
+        printf( " failed\n  ! ssl_set_own_cert returned %d\n\n", ret );
+        goto exit;
+    }
 
 #if defined(POLARSSL_SSL_SERVER_NAME_INDICATION)
-    ssl_set_hostname( &ssl, opt.server_name );
+    if( ( ret = ssl_set_hostname( &ssl, opt.server_name ) ) != 0 )
+    {
+        printf( " failed\n  ! ssl_set_hostname returned %d\n\n", ret );
+        goto exit;
+    }
 #endif
 
     if( opt.mode == MODE_SSL_TLS )
@@ -710,8 +719,13 @@ int main( int argc, char *argv[] )
         fflush( stdout );
 
         n = sizeof( buf );
-        len = base64_encode( base, &n, (const unsigned char *) opt.user_name,
+        ret = base64_encode( base, &n, (const unsigned char *) opt.user_name,
                              strlen( opt.user_name ) );
+
+        if( ret != 0 ) {
+            printf( " failed\n  ! base64_encode returned %d\n\n", ret );
+            goto exit;
+        }
         len = sprintf( (char *) buf, "%s\r\n", base );
         ret = write_ssl_and_get_response( &ssl, buf, len );
         if( ret < 300 || ret > 399 )
@@ -725,8 +739,13 @@ int main( int argc, char *argv[] )
         printf( "  > Write password to server: %s", opt.user_pwd );
         fflush( stdout );
 
-        len = base64_encode( base, &n, (const unsigned char *) opt.user_pwd,
+        ret = base64_encode( base, &n, (const unsigned char *) opt.user_pwd,
                              strlen( opt.user_pwd ) );
+
+        if( ret != 0 ) {
+            printf( " failed\n  ! base64_encode returned %d\n\n", ret );
+            goto exit;
+        }
         len = sprintf( (char *) buf, "%s\r\n", base );
         ret = write_ssl_and_get_response( &ssl, buf, len );
         if( ret < 200 || ret > 399 )
@@ -808,6 +827,7 @@ exit:
     x509_crt_free( &cacert );
     pk_free( &pkey );
     ssl_free( &ssl );
+    ctr_drbg_free( &ctr_drbg );
     entropy_free( &entropy );
 
 #if defined(_WIN32)
