@@ -129,6 +129,10 @@ static void *handle_ssl_connection( void *data )
     ssl_context ssl;
     ctr_drbg_context ctr_drbg;
 
+    /* Make sure memory references are valid */
+    memset( &ssl, 0, sizeof( ssl_context ) );
+    memset( &ctr_drbg, 0, sizeof( ctr_drbg_context ) );
+
     snprintf( pers, sizeof(pers), "SSL Pthread Thread %d", thread_id );
     printf( "  [ #%d ]  Client FD %d\n", thread_id, client_fd );
     printf( "  [ #%d ]  Seeding the random number generator...\n", thread_id );
@@ -173,7 +177,11 @@ static void *handle_ssl_connection( void *data )
 #endif
 
     ssl_set_ca_chain( &ssl, thread_info->ca_chain, NULL, NULL );
-    ssl_set_own_cert( &ssl, thread_info->server_cert, thread_info->server_key );
+    if( ( ret = ssl_set_own_cert( &ssl, thread_info->server_cert, thread_info->server_key ) ) != 0 )
+    {
+        printf( " failed\n  ! ssl_set_own_cert returned %d\n\n", ret );
+        goto thread_exit;
+    }
 
     printf( "  [ #%d ]  ok\n", thread_id );
 
@@ -232,8 +240,6 @@ static void *handle_ssl_connection( void *data )
                             thread_id, -ret );
                     goto thread_exit;
             }
-
-            break;
         }
 
         len = ret;
@@ -304,6 +310,7 @@ thread_exit:
 #endif
 
     net_close( client_fd );
+    ctr_drbg_free( &ctr_drbg );
     ssl_free( &ssl );
 
     thread_info->thread_complete = 1;
