@@ -32,10 +32,12 @@
 #include <string.h>
 #include <stdio.h>
 
+#include "polarssl/entropy.h"
 #include "polarssl/hmac_drbg.h"
 #include "polarssl/ctr_drbg.h"
 #include "polarssl/dhm.h"
 #include "polarssl/gcm.h"
+#include "polarssl/ccm.h"
 #include "polarssl/md2.h"
 #include "polarssl/md4.h"
 #include "polarssl/md5.h"
@@ -132,11 +134,16 @@ int main( int argc, char *argv[] )
         return( ret );
 #endif
 
-#if defined(POLARSSL_GCM_C)
+#if defined(POLARSSL_GCM_C) && defined(POLARSSL_AES_C)
     if( ( ret = gcm_self_test( v ) ) != 0 )
         return( ret );
 #endif
 
+#if defined(POLARSSL_CCM_C) && defined(POLARSSL_AES_C)
+    if( ( ret = ccm_self_test( v ) ) != 0 )
+        return( ret );
+#endif
+
 #if defined(POLARSSL_BASE64_C)
     if( ( ret = base64_self_test( v ) ) != 0 )
         return( ret );
@@ -187,6 +194,11 @@ int main( int argc, char *argv[] )
         return( ret );
 #endif
 
+#if defined(POLARSSL_ENTROPY_C)
+    if( ( ret = entropy_self_test( v ) ) != 0 )
+        return( ret );
+#endif
+
 /* Slow tests last */
 
 #if defined(POLARSSL_PBKDF2_C)
@@ -199,10 +211,8 @@ int main( int argc, char *argv[] )
 #endif
 #endif
 
-/*
- * Not reliable enough yet
- */
-#if 0 && defined(POLARSSL_TIMING_C)
+/* Not stable enough on Windows and FreeBSD yet */
+#if __linux__ && defined(POLARSSL_TIMING_C)
     if( ( ret = timing_self_test( v ) ) != 0 )
         return( ret );
 #endif
