@@ -142,7 +142,7 @@ static void my_debug( void *ctx, int level, const char *str )
  */
 static int ssl_test( struct options *opt )
 {
-    int ret, i;
+    int ret = 1, i;
     int client_fd = -1;
     int bytes_to_read;
     int bytes_to_write;
@@ -169,8 +169,6 @@ static int ssl_test( struct options *opt )
     x509_crt srvcert;
     pk_context pkey;
 
-    ret = 1;
-
     memset( &ssl, 0, sizeof(ssl_context) );
     entropy_init( &entropy );
     x509_crt_init( &srvcert );
@@ -265,7 +263,11 @@ static int ssl_test( struct options *opt )
 
         ssl_set_endpoint( &ssl, SSL_IS_SERVER );
         ssl_set_ca_chain( &ssl, srvcert.next, NULL, NULL );
-        ssl_set_own_cert( &ssl, &srvcert, &pkey );
+        if( ( ret = ssl_set_own_cert( &ssl, &srvcert, &pkey ) ) != 0 )
+        {
+            printf( " failed\n  ! ssl_set_own_cert returned %d\n\n", ret );
+            goto exit;
+        }
     }
 
     ssl_set_authmode( &ssl, SSL_VERIFY_NONE );
@@ -413,6 +415,7 @@ exit:
     x509_crt_free( &srvcert );
     pk_free( &pkey );
     ssl_free( &ssl );
+    ctr_drbg_free( &ctr_drbg );
     entropy_free( &entropy );
 
     if( client_fd != -1 )
