@@ -1,24 +1,21 @@
-#!/bin/bash
+#!/bin/sh
 
 # Test interop with OpenSSL and GnuTLS (and self-op while at it).
 #
 # Check each common ciphersuite, with each version, both ways (client/server),
 # with and without client authentication.
+#
+# Peer version requirements:
+# - OpenSSL 1.0.1e 11 Feb 2013 (probably since 1.0.1, tested with 1.0.1e)
+# - GnuTLS 3.2.15 (probably works since 3.2.12 but tested only with 3.2.15)
 
 set -u
 
-# test if those two are set in the environment before assigning defaults
-if [ -n "${GNUTLS_CLI:-}" -a -n "${GNUTLS_SERV:-}" ]; then
-    GNUTLS_AVAILABLE=1
-else
-    GNUTLS_AVAILABLE=0
-fi
-
 # initialise counters
-let "tests = 0"
-let "failed = 0"
-let "skipped = 0"
-let "srvmem = 0"
+TESTS=0
+FAILED=0
+SKIPPED=0
+SRVMEM=0
 
 # default commands, can be overriden by the environment
 : ${P_SRV:=../programs/ssl/ssl_server2}
@@ -27,6 +24,21 @@ let "srvmem = 0"
 : ${GNUTLS_CLI:=gnutls-cli}
 : ${GNUTLS_SERV:=gnutls-serv}
 
+# do we have a recent enough GnuTLS?
+if ( which $GNUTLS_CLI && which $GNUTLS_SERV ) >/dev/null; then
+    eval $( $GNUTLS_CLI --version | head -n1 | sed 's/.* \([0-9]*\)\.\([0-9]\)*\.\([0-9]*\)$/MAJOR="\1" MINOR="\2" PATCH="\3"/' )
+    if [ $MAJOR -lt 3 -o \
+        \( $MAJOR -eq 3 -a $MINOR -lt 2 \) -o \
+        \( $MAJOR -eq 3 -a $MINOR -eq 2 -a $PATCH -lt 15 \) ]
+    then
+        PEER_GNUTLS=""
+    else
+        PEER_GNUTLS=" GnuTLS"
+    fi
+else
+    PEER_GNUTLS=""
+fi
+
 # default values for options
 MODES="ssl3 tls1 tls1_1 tls1_2"
 VERIFIES="NO YES"
@@ -35,12 +47,7 @@ FILTER=""
 EXCLUDE='NULL\|DES-CBC-' # avoid plain DES but keep 3DES-EDE-CBC (PolarSSL), DES-CBC3 (OpenSSL)
 VERBOSE=""
 MEMCHECK=0
-# GnuTLS not enabled by default, 3.2.4 might not be available everywhere
-if [ "$GNUTLS_AVAILABLE" -gt 0 ]; then
-    PEERS="OpenSSL PolarSSL GnuTLS"
-else
-    PEERS="OpenSSL PolarSSL"
-fi
+PEERS="OpenSSL$PEER_GNUTLS PolarSSL"
 
 print_usage() {
     echo "Usage: $0"
@@ -51,7 +58,7 @@ print_usage() {
     echo -e "  -t|--types\tWhich key exchange type to perform (Default: '$TYPES')"
     echo -e "  -V|--verify\tWhich verification modes to perform (Default: '$VERIFIES')"
     echo -e "  -p|--peers\tWhich peers to use (Default: '$PEERS')"
-    echo -e "            \tAlso available: GnuTLS (needs v3.2.4 or higher)"
+    echo -e "            \tAlso available: GnuTLS (needs v3.2.15 or higher)"
     echo -e "  -M|--memcheck\tCheck memory leaks and errors."
     echo -e "  -v|--verbose\tSet verbose output."
 }
@@ -202,7 +209,6 @@ add_common_ciphersuites()
                 TLS-RSA-WITH-RC4-128-MD5                \
                 TLS-RSA-WITH-NULL-MD5                   \
                 TLS-RSA-WITH-NULL-SHA                   \
-                TLS-RSA-WITH-NULL-SHA256                \
                 "
             G_CIPHERS="$G_CIPHERS                       \
                 +DHE-RSA:+AES-128-CBC:+SHA1             \
@@ -219,7 +225,6 @@ add_common_ciphersuites()
                 +RSA:+ARCFOUR-128:+MD5                  \
                 +RSA:+NULL:+MD5                         \
                 +RSA:+NULL:+SHA1                        \
-                +RSA:+NULL:+SHA256                      \
                 "
             O_CIPHERS="$O_CIPHERS               \
                 DHE-RSA-AES128-SHA              \
@@ -410,6 +415,15 @@ add_gnutls_ciphersuites()
             ;;
 
         "RSA")
+            if [ "$MODE" != "ssl3" ];
+            then
+                P_CIPHERS="$P_CIPHERS                           \
+                    TLS-RSA-WITH-NULL-SHA256                    \
+                    "
+                G_CIPHERS="$G_CIPHERS                           \
+                    +RSA:+NULL:+SHA256                          \
+                    "
+            fi
             if [ "$MODE" = "tls1_2" ];
             then
                 P_CIPHERS="$P_CIPHERS                           \
@@ -444,30 +458,39 @@ add_gnutls_ciphersuites()
             ;;
 
         "PSK")
-            # GnuTLS 3.2.11 (2014-02-13) requires TLS 1.x for most *PSK suites
+            P_CIPHERS="$P_CIPHERS                               \
+                TLS-DHE-PSK-WITH-3DES-EDE-CBC-SHA               \
+                TLS-DHE-PSK-WITH-AES-128-CBC-SHA                \
+                TLS-DHE-PSK-WITH-AES-256-CBC-SHA                \
+                TLS-DHE-PSK-WITH-RC4-128-SHA                    \
+                "
+            G_CIPHERS="$G_CIPHERS                               \
+                +DHE-PSK:+3DES-CBC:+SHA1                        \
+                +DHE-PSK:+AES-128-CBC:+SHA1                     \
+                +DHE-PSK:+AES-256-CBC:+SHA1                     \
+                +DHE-PSK:+ARCFOUR-128:+SHA1                     \
+                "
             if [ "$MODE" != "ssl3" ];
             then
                 P_CIPHERS="$P_CIPHERS                           \
                     TLS-ECDHE-PSK-WITH-AES-256-CBC-SHA          \
                     TLS-ECDHE-PSK-WITH-AES-128-CBC-SHA          \
                     TLS-ECDHE-PSK-WITH-3DES-EDE-CBC-SHA         \
-                    TLS-DHE-PSK-WITH-3DES-EDE-CBC-SHA           \
-                    TLS-DHE-PSK-WITH-AES-128-CBC-SHA            \
-                    TLS-DHE-PSK-WITH-AES-256-CBC-SHA            \
+                    TLS-ECDHE-PSK-WITH-RC4-128-SHA              \
                     TLS-RSA-PSK-WITH-3DES-EDE-CBC-SHA           \
                     TLS-RSA-PSK-WITH-AES-256-CBC-SHA            \
                     TLS-RSA-PSK-WITH-AES-128-CBC-SHA            \
+                    TLS-RSA-PSK-WITH-RC4-128-SHA                \
                     "
                 G_CIPHERS="$G_CIPHERS                           \
-                    +ECDHE-PSK:+AES-256-CBC:+SHA1               \
-                    +ECDHE-PSK:+AES-128-CBC:+SHA1               \
                     +ECDHE-PSK:+3DES-CBC:+SHA1                  \
-                    +DHE-PSK:+3DES-CBC:+SHA1                    \
-                    +DHE-PSK:+AES-128-CBC:+SHA1                 \
-                    +DHE-PSK:+AES-256-CBC:+SHA1                 \
+                    +ECDHE-PSK:+AES-128-CBC:+SHA1               \
+                    +ECDHE-PSK:+AES-256-CBC:+SHA1               \
+                    +ECDHE-PSK:+ARCFOUR-128:+SHA1               \
                     +RSA-PSK:+3DES-CBC:+SHA1                    \
                     +RSA-PSK:+AES-256-CBC:+SHA1                 \
                     +RSA-PSK:+AES-128-CBC:+SHA1                 \
+                    +RSA-PSK:+ARCFOUR-128:+SHA1                 \
                     "
             fi
             if [ "$MODE" = "tls1_2" ];
@@ -570,25 +593,54 @@ add_polarssl_ciphersuites()
                 P_CIPHERS="$P_CIPHERS                               \
                     TLS-ECDH-ECDSA-WITH-CAMELLIA-128-GCM-SHA256     \
                     TLS-ECDH-ECDSA-WITH-CAMELLIA-256-GCM-SHA384     \
+                    TLS-ECDHE-ECDSA-WITH-AES-128-CCM                \
+                    TLS-ECDHE-ECDSA-WITH-AES-256-CCM                \
+                    TLS-ECDHE-ECDSA-WITH-AES-128-CCM-8              \
+                    TLS-ECDHE-ECDSA-WITH-AES-256-CCM-8              \
                     "
             fi
             ;;
 
         "RSA")
+            if [ "$MODE" = "tls1_2" ];
+            then
+                P_CIPHERS="$P_CIPHERS                               \
+                    TLS-RSA-WITH-AES-128-CCM                        \
+                    TLS-RSA-WITH-AES-256-CCM                        \
+                    TLS-DHE-RSA-WITH-AES-128-CCM                    \
+                    TLS-DHE-RSA-WITH-AES-256-CCM                    \
+                    TLS-RSA-WITH-AES-128-CCM-8                      \
+                    TLS-RSA-WITH-AES-256-CCM-8                      \
+                    TLS-DHE-RSA-WITH-AES-128-CCM-8                  \
+                    TLS-DHE-RSA-WITH-AES-256-CCM-8                  \
+                    "
+            fi
             ;;
 
         "PSK")
+            # *PKS-NULL-SHA suites supported by GnuTLS 3.3.5 but not 3.2.15
             P_CIPHERS="$P_CIPHERS                        \
                 TLS-PSK-WITH-NULL-SHA                    \
-                TLS-DHE-PSK-WITH-RC4-128-SHA             \
                 TLS-DHE-PSK-WITH-NULL-SHA                \
-                TLS-RSA-PSK-WITH-RC4-128-SHA             \
                 "
             if [ "$MODE" != "ssl3" ];
             then
                 P_CIPHERS="$P_CIPHERS                    \
-                    TLS-ECDHE-PSK-WITH-RC4-128-SHA       \
                     TLS-ECDHE-PSK-WITH-NULL-SHA          \
+                    TLS-RSA-PSK-WITH-NULL-SHA            \
+                    "
+            fi
+            if [ "$MODE" = "tls1_2" ];
+            then
+                P_CIPHERS="$P_CIPHERS                               \
+                    TLS-PSK-WITH-AES-128-CCM                        \
+                    TLS-PSK-WITH-AES-256-CCM                        \
+                    TLS-DHE-PSK-WITH-AES-128-CCM                    \
+                    TLS-DHE-PSK-WITH-AES-256-CCM                    \
+                    TLS-PSK-WITH-AES-128-CCM-8                      \
+                    TLS-PSK-WITH-AES-256-CCM-8                      \
+                    TLS-DHE-PSK-WITH-AES-128-CCM-8                  \
+                    TLS-DHE-PSK-WITH-AES-256-CCM-8                  \
                     "
             fi
             ;;
@@ -615,14 +667,14 @@ setup_arguments()
             exit 1;
     esac
 
-    P_SERVER_ARGS="server_addr=0.0.0.0 force_version=$MODE"
-    O_SERVER_ARGS="-www -cipher NULL,ALL -$MODE"
-    G_SERVER_ARGS="-p 4433 --http"
+    P_SERVER_ARGS="server_port=$PORT server_addr=0.0.0.0 force_version=$MODE"
+    O_SERVER_ARGS="-accept $PORT -www -cipher NULL,ALL -$MODE"
+    G_SERVER_ARGS="-p $PORT --http"
     G_SERVER_PRIO="EXPORT:+NULL:+MD5:+PSK:+DHE-PSK:+ECDHE-PSK:+RSA-PSK:-VERS-TLS-ALL:$G_PRIO_MODE"
 
-    P_CLIENT_ARGS="force_version=$MODE"
-    O_CLIENT_ARGS="-$MODE"
-    G_CLIENT_ARGS="-p 4433 --debug 3"
+    P_CLIENT_ARGS="server_port=$PORT force_version=$MODE"
+    O_CLIENT_ARGS="-connect localhost:$PORT -$MODE"
+    G_CLIENT_ARGS="-p $PORT --debug 3"
     G_CLIENT_PRIO="NONE:$G_PRIO_MODE:+COMP-NULL:+CURVE-ALL:+SIGN-ALL"
 
     if [ "X$VERIFY" = "XYES" ];
@@ -727,62 +779,63 @@ start_server() {
     SERVER_NAME=$1
 
     log "$SERVER_CMD"
-    echo "$SERVER_CMD" > srv_out
-    $SERVER_CMD >> srv_out 2>&1 &
+    echo "$SERVER_CMD" > $SRV_OUT
+    $SERVER_CMD >> $SRV_OUT 2>&1 &
     PROCESS_ID=$!
 
     sleep 1
 }
 
-# terminate the running server (closing it cleanly if it is ours)
+# terminate the running server
 stop_server() {
-    case $SERVER_NAME in
-        [Pp]olar*)
-            # we must force a PSK suite when in PSK mode (otherwise client
-            # auth will fail), so try every entry in $P_CIPHERS in turn (in
-            # case the first one is not implemented in this configuration)
-            for i in $P_CIPHERS; do
-                log "$P_CLI $P_CLIENT_ARGS request_page=SERVERQUIT auth_mode=none force_ciphersuite=$i"
-                "$P_CLI" $P_CLIENT_ARGS request_page=SERVERQUIT auth_mode=none \
-                    force_ciphersuite=$i >/dev/null
-                if [ "$?" == 0 ]; then
-                    break
-                fi
-            done
-            ;;
-        *)
-            kill $PROCESS_ID 2>/dev/null
-    esac
-
+    kill $PROCESS_ID 2>/dev/null
     wait $PROCESS_ID 2>/dev/null
 
     if [ "$MEMCHECK" -gt 0 ]; then
-        if is_polar "$SERVER_CMD" && has_mem_err srv_out; then
+        if is_polar "$SERVER_CMD" && has_mem_err $SRV_OUT; then
             echo "  ! Server had memory errors"
-            let "srvmem++"
+            SRVMEM=$(( $SRVMEM + 1 ))
             return
         fi
     fi
 
-    rm -f srv_out
+    rm -f $SRV_OUT
 }
 
 # kill the running server (used when killed by signal)
 cleanup() {
-    rm -f srv_out cli_out
-    kill $PROCESS_ID
+    rm -f $SRV_OUT $CLI_OUT
+    kill $PROCESS_ID >/dev/null 2>&1
+    kill $WATCHDOG_PID >/dev/null 2>&1
     exit 1
 }
 
+# wait for client to terminate and set EXIT
+# must be called right after starting the client
+wait_client_done() {
+    CLI_PID=$!
+
+    ( sleep "$DOG_DELAY"; echo "TIMEOUT" >> $CLI_OUT; kill $CLI_PID ) &
+    WATCHDOG_PID=$!
+
+    wait $CLI_PID
+    EXIT=$?
+
+    kill $WATCHDOG_PID
+    wait $WATCHDOG_PID
+
+    echo "EXIT: $EXIT" >> $CLI_OUT
+}
+
 # run_client <name> <cipher>
 run_client() {
     # announce what we're going to do
-    let "tests++"
+    TESTS=$(( $TESTS + 1 ))
     VERIF=$(echo $VERIFY | tr '[:upper:]' '[:lower:]')
-    TITLE="${1:0:1}->${SERVER_NAME:0:1} $MODE,$VERIF $2"
+    TITLE="`echo $1 | head -c1`->`echo $SERVER_NAME | head -c1`"
+    TITLE="$TITLE $MODE,$VERIF $2"
     echo -n "$TITLE "
-    LEN=`echo "$TITLE" | wc -c`
-    LEN=`echo 72 - $LEN | bc`
+    LEN=$(( 72 - `echo "$TITLE" | wc -c` ))
     for i in `seq 1 $LEN`; do echo -n '.'; done; echo -n ' '
 
     # run the command and interpret result
@@ -790,14 +843,14 @@ run_client() {
         [Oo]pen*)
             CLIENT_CMD="$OPENSSL_CMD s_client $O_CLIENT_ARGS -cipher $2"
             log "$CLIENT_CMD"
-            echo "$CLIENT_CMD" > cli_out
-            ( echo -e 'GET HTTP/1.0'; echo; ) | $CLIENT_CMD >> cli_out 2>&1
-            EXIT=$?
+            echo "$CLIENT_CMD" > $CLI_OUT
+            ( echo -e 'GET HTTP/1.0'; echo; ) | $CLIENT_CMD >> $CLI_OUT 2>&1 &
+            wait_client_done
 
-            if [ "$EXIT" == "0" ]; then
+            if [ $EXIT -eq 0 ]; then
                 RESULT=0
             else
-                if grep 'Cipher is (NONE)' cli_out >/dev/null; then
+                if grep 'Cipher is (NONE)' $CLI_OUT >/dev/null; then
                     RESULT=1
                 else
                     RESULT=2
@@ -808,18 +861,18 @@ run_client() {
         [Gg]nu*)
             CLIENT_CMD="$GNUTLS_CLI $G_CLIENT_ARGS --priority $G_PRIO_MODE:$2 localhost"
             log "$CLIENT_CMD"
-            echo "$CLIENT_CMD" > cli_out
-            ( echo -e 'GET HTTP/1.0'; echo; ) | $CLIENT_CMD >> cli_out 2>&1
-            EXIT=$?
+            echo "$CLIENT_CMD" > $CLI_OUT
+            ( echo -e 'GET HTTP/1.0'; echo; ) | $CLIENT_CMD >> $CLI_OUT 2>&1 &
+            wait_client_done
 
-            if [ "$EXIT" == "0" ]; then
+            if [ $EXIT -eq 0 ]; then
                 RESULT=0
             else
                 RESULT=2
                 # interpret early failure, with a handshake_failure alert
                 # before the server hello, as "no ciphersuite in common"
-                if grep -F 'Received alert [40]: Handshake failed' cli_out; then
-                    if grep -i 'SERVER HELLO .* was received' cli_out; then :
+                if grep -F 'Received alert [40]: Handshake failed' $CLI_OUT; then
+                    if grep -i 'SERVER HELLO .* was received' $CLI_OUT; then :
                     else
                         RESULT=1
                     fi
@@ -833,9 +886,9 @@ run_client() {
                 CLIENT_CMD="valgrind --leak-check=full $CLIENT_CMD"
             fi
             log "$CLIENT_CMD"
-            echo "$CLIENT_CMD" > cli_out
-            $CLIENT_CMD >> cli_out 2>&1
-            EXIT=$?
+            echo "$CLIENT_CMD" > $CLI_OUT
+            $CLIENT_CMD >> $CLI_OUT 2>&1 &
+            wait_client_done
 
             case $EXIT in
                 "0")    RESULT=0    ;;
@@ -844,7 +897,7 @@ run_client() {
             esac
 
             if [ "$MEMCHECK" -gt 0 ]; then
-                if is_polar "$CLIENT_CMD" && has_mem_err cli_out; then
+                if is_polar "$CLIENT_CMD" && has_mem_err $CLI_OUT; then
                     RESULT=2
                 fi
             fi
@@ -857,7 +910,7 @@ run_client() {
             ;;
     esac
 
-    echo "EXIT: $EXIT" >> cli_out
+    echo "EXIT: $EXIT" >> $CLI_OUT
 
     # report and count result
     case $RESULT in
@@ -866,18 +919,27 @@ run_client() {
             ;;
         "1")
             echo SKIP
-            let "skipped++"
+            SKIPPED=$(( $SKIPPED + 1 ))
             ;;
         "2")
             echo FAIL
-            cp srv_out c-srv-${tests}.log
-            cp cli_out c-cli-${tests}.log
-            echo "  ! outputs saved to c-srv-${tests}.log, c-cli-${tests}.log"
-            let "failed++"
+            cp $SRV_OUT c-srv-${TESTS}.log
+            cp $CLI_OUT c-cli-${TESTS}.log
+            echo "  ! outputs saved to c-srv-${TESTS}.log, c-cli-${TESTS}.log"
+
+            if [ "X${USER:-}" = Xbuildbot -o "X${LOGNAME:-}" = Xbuildbot ]; then
+                echo "  ! server output:"
+                cat c-srv-${TESTS}.log
+                echo "  ! ==================================================="
+                echo "  ! client output:"
+                cat c-cli-${TESTS}.log
+            fi
+
+            FAILED=$(( $FAILED + 1 ))
             ;;
     esac
 
-    rm -f cli_out
+    rm -f $CLI_OUT
 }
 
 #
@@ -922,7 +984,21 @@ for PEER in $PEERS; do
     esac
 done
 
-killall -q gnutls-serv openssl ssl_server ssl_server2
+# Pick a "unique" port in the range 10000-19999.
+PORT="0000$$"
+PORT="1$(echo $PORT | tail -c 5)"
+
+# Also pick a unique name for intermediate files
+SRV_OUT="srv_out.$$"
+CLI_OUT="cli_out.$$"
+
+# client timeout delay: be more patient with valgrind
+if [ "$MEMCHECK" -gt 0 ]; then
+    DOG_DELAY=30
+else
+    DOG_DELAY=10
+fi
+
 trap cleanup INT TERM HUP
 
 for VERIFY in $VERIFIES; do
@@ -1004,7 +1080,7 @@ for VERIFY in $VERIFIES; do
                     ;;
 
                 *)
-                    echo "Unkown peer: $PEER" >&2
+                    echo "Unknown peer: $PEER" >&2
                     exit 1
                     ;;
 
@@ -1017,7 +1093,7 @@ done
 
 echo "------------------------------------------------------------------------"
 
-if (( failed != 0 || srvmem != 0 ));
+if [ $FAILED -ne 0 -o $SRVMEM -ne 0 ];
 then
     echo -n "FAILED"
 else
@@ -1025,13 +1101,13 @@ else
 fi
 
 if [ "$MEMCHECK" -gt 0 ]; then
-    MEMREPORT=", $srvmem server memory errors"
+    MEMREPORT=", $SRVMEM server memory errors"
 else
     MEMREPORT=""
 fi
 
-let "passed = tests - failed"
-echo " ($passed / $tests tests ($skipped skipped$MEMREPORT))"
+PASSED=$(( $TESTS - $FAILED ))
+echo " ($PASSED / $TESTS tests ($SKIPPED skipped$MEMREPORT))"
 
-let "failed += srvmem"
-exit $failed
+FAILED=$(( $FAILED + $SRVMEM ))
+exit $FAILED
